@@ -74,19 +74,6 @@ def initLogger(level_color=None, format_color=None, log_format=None,
     :return: logger，使用该logger进行日志输出。
     """
 
-    # def sink(msg):
-    #     print(msg)
-    #     record = msg.record
-    #     file = record["file"].name
-    #     function = record["function"]
-    #     line = record["line"]
-    #     message = record['message']
-    #     module = record["module"]
-    #     name = record["name"]
-    #
-    #     if record.level.no == DEBUG:
-    #         pass
-
     def setLevel(logger_cls, *args, **kwargs):
         logger.debug("暂不支持中途更改日志级别")
 
@@ -153,69 +140,6 @@ def initLogger(level_color=None, format_color=None, log_format=None,
     return logger
 
 
-def holdLoggingLevel(para='end', outer=True):
-    """
-    临时提高日志级别，例如更改前使用holdLoggingLevel(logging.WARN)将日志级别临时更改为“WARN”，
-    更改后调用holdLoggingLevel('end')恢复日志级别为临时更改前的级别。
-    holdLoggingLevel(level)和holdLoggingLevel('end')构成一个区间。
-    多个区间可以嵌套，以最外层区间的设置为最终设置。如果需要以内部区间的设置为最终设置，将outer设为False即可。
-
-    outer=False和outer=True优先级相同，后设置的生效。
-
-    使用示例：
-
-    ...any code...                                             \n
-    holdLoggingLevel(logging.DEBUG, False)  # 1                \n
-    ...DEBUG生效...                                            \n
-    holdLoggingLevel('logging.WARN, False)  # 2                \n
-    ...WARN生效...                                              \n
-    holdLoggingLevel('logging.DEBUG, False)  # 3                \n
-    ...DEBUG生效...                                             \n
-    holdLoggingLevel('logging.WARN')  # 4                       \n
-    ...WARN生效...                                              \n
-    holdLoggingLevel('logging.DEBUG')  # 5                      \n
-    ...WARN生效...，因为outer=True，以当前设置更外层的为准       \n
-    holdLoggingLevel('logging.ERROR')  # 6                      \n
-    ...WARN生效...，因为outer=True，一直找到最外层的日志级别      \n
-    holdLoggingLevel('end')  # 关闭6                            \n
-    ...WARN生效...                                              \n
-    holdLoggingLevel('end')  # 关闭5                            \n
-    ...WARN生效...                                              \n
-    holdLoggingLevel('end')  # 关闭4                            \n
-    ...DEBUG生效...，因为4对应的设置已经关闭，则以3为准           \n
-    holdLoggingLevel('end',outer=False)                        \n
-    ...WARN生效...，因为3对应的设置已经关闭，则以2为准            \n
-    holdLoggingLevel('end',outer=False)                        \n
-    ...DEBUG生效...                                            \n
-    ...DEBUG生效...如果outer=False，且与之对应的end语句一直不出现，则级别设置一直到程序结束都生效
-    """
-    # global level
-    # global prior  # 区间嵌套层数
-    # if outer:  # 如果外部优先
-    #     if para in [DEBUG, INFO, WARN, ERROR, CRITICAL]:
-    #         prior += 1
-    #         if prior == 1:  # 如果是最外层就操作
-    #             level = logger.getEffectiveLevel()  # 记录临时修改前日志级别
-    #             logger.setLevel(para)
-    #     elif type(para) == str and para.lower() == 'end':
-    #         # 如果第一个变量为False，则不会判断第二个变量，所以即使para不是字符串，para.lower()不会报错
-    #         prior -= 1
-    #         if prior == 0:
-    #             logger.setLevel(level)
-    #     else:
-    #         logger.error('参数未识别，para只能取值logging日志的有效级别或字符串"end"!')
-    # else:  # 如果内部优先
-    #     if para in [logging.DEBUG, logging.INFO, logging.WARN, logging.ERROR, logging.CRITICAL]:
-    #         level = logger.getEffectiveLevel()
-    #         logger.setLevel(para)
-    #     elif type(para) == str and para.lower() == 'end':
-    #         # 恢复日志级别为外部级别
-    #         logger.setLevel(level)
-    #     else:
-    #         logger.error('参数未识别，para只能取值logging日志的有效级别或字符串"end"!')
-    pass
-
-
 def smart_decorator(decorator):
     """
     智能装饰器，用于装饰装饰器，使得装饰器无论在有没有参数的情况下可以使用同样的语法。无论是类装饰器和方法装饰其都可以装饰。
@@ -280,31 +204,6 @@ def loggingTitleCall(func, title=None):
     return wrap_f
 
 
-@smart_decorator
-def SqliteClass(cls, table=None):
-    """
-    一个装饰器，在其他类上标记@SqliteClass，使得其他类可以用于和sqlite互转
-
-    :param cls:
-    :param table:
-    :return:
-    """
-    cls.__tablename__ = 'user'
-    columns = [*cls.__init__.__code__.co_names]
-    sig = inspect.signature(cls.__init__)
-    for par in sig.parameters.values():
-        if par.annotation is not inspect._empty:
-            print('Parameter: ', par)
-            print("name: ", par.name)
-            print("type: ", par.annotation)
-
-    class NewClass():
-        def __init__(self, *args, **kwargs):
-            self.new = cls(*args, **kwargs)
-
-    return NewClass
-
-
 def loggingTitle(title: str = "default", logger=None, start_end: str = "start"):
     """
     输出以下格式的日志标题
@@ -427,26 +326,47 @@ def remove_file_logger():
         logger.remove(log_file_id)
 
 
-def add_file_logger(log_file: Optional[str] = "default", rotation="50 MB", retention="1 days", diagnose=True):
+def add_file_logger(log_file: Optional[str] = "default", rotation="50 MB", retention="10 files", diagnose=True):
     """
+    添加文件日志处理器
 
-    :param diagnose: 运行出错时，是否追踪错误原因
-    :param retention: 文件日志输出时，指定保留日志的期限，如“30 days”表示只保留最近30天的日志
-    :param rotation: 文件日志输出时，输出下一个日志的条件，可以是大小，也可以是时间，如"50 MB"、"00:00", "1 week"等，具体参见loguru
-    :param log_file:
-    :return:
+    :param log_file: 日志文件路径，为None时不输出到文件
+    :param rotation: 日志轮转条件，"50 MB": 文件大小达到50MB时轮转，"00:00": 每天午夜轮转，"1 week": 每周轮转，"100": 每100条日志轮转
+    :param retention: 日志保留期限，如"30 days": 保留最近30天的日志，"1 week": 保留最近1周的日志，"10 files": 最多保留10个日志文件
+    :param diagnose: 控制是否在异常时显示详细的诊断信息，当为 True 时，会在日志中包含异常的详细上下文信息，否则，只显示基本的异常信息
+    :return: 日志文件处理器ID
     """
     global logger, log_file_id
     settings = get_settings()
-    if settings.get_settings("logger.logFile") == "None":
-        log_file = None
-    level = settings.get_settings("logger.level") or 10
-    if log_file == "default":
-        log_file = "runtime_{time}.log"
 
+    # 优先使用传入的log_file参数
+    if log_file == "default":
+        # 如果没有传入log_file，则从settings中读取
+        settings_log_file = settings.get_settings("logger.logFile")
+        if settings_log_file == "None" or settings_log_file is None:
+            log_file = None
+        else:
+            log_file = settings_log_file
+    elif log_file is None:
+        # 显式传入None，不输出到文件
+        log_file = None
+    # 如果传入了具体的文件路径，则直接使用
+
+    # 获取日志级别
+    level = settings.get_settings("logger.level") or 10
+
+    # 添加文件日志处理器
     if log_file is not None:
-        log_file_id = logger.add(log_file, encoding="utf8", level=level, rotation=rotation, retention=retention,
-                                 diagnose=diagnose)
+        log_file_id = logger.add(
+            log_file,
+            encoding="utf8",
+            level=level,
+            rotation=rotation,
+            retention=retention,
+            diagnose=diagnose
+        )
+        return log_file_id
+    return None
 
 
 def __import_first__():
@@ -461,5 +381,4 @@ def __import_first__():
                         date_format=dateFormat)
 
 
-prior = 0  # 用来控制临时修改日志级别的变量
 __import_first__()
