@@ -197,6 +197,7 @@ def read_from_pickle(file: str):
             obj = pickle.load(f)
     except:
         traceback.print_exc()
+        logger.error(f"加载文件{f}时发生错误！")
     return obj
 
 
@@ -253,13 +254,14 @@ def write_excel(file, dataframe_dict: dict, index=False):
     return
 
 
-def read_excel(file, need_merge=False, index_col=True):
+def read_excel(file, need_merge=False, index_col=True, header=0):
     """
     将excel文件中的所有sheet读取到dataframe_dict中
 
     :param need_merge: 是否合并各个sheet中的数据，默认不合并，且只有各个sheet中数据格式相同时才能合并
     :param file:
     :param index_col: 是否需要索引，也可以是数字
+    :param header: 设置表头
     :return:
     """
     if index_col:
@@ -269,7 +271,7 @@ def read_excel(file, need_merge=False, index_col=True):
     if not need_merge:
         dataframe_dict = {}
         for sheet in sheets:
-            df = pd.read_excel(file, sheet_name=sheet, index_col=index_col)
+            df = pd.read_excel(file, sheet_name=sheet, index_col=index_col, header=header)
             dataframe_dict.update({sheet: df})
         return dataframe_dict
     else:
@@ -277,9 +279,10 @@ def read_excel(file, need_merge=False, index_col=True):
         res = None
         for sheet in sheets:
             if res is None:
-                res = pd.read_excel(file, sheet_name=sheet, index_col=index_col)
+                res = pd.read_excel(file, sheet_name=sheet, index_col=index_col, header=header)
             else:
-                res = merge_dataframes_simple([res, pd.read_excel(file, sheet_name=sheet, index_col=index_col)])
+                res = merge_dataframes_simple(
+                    [res, pd.read_excel(file, sheet_name=sheet, header=header, index_col=index_col)])
 
         return res
 
@@ -311,9 +314,9 @@ def read_csv_ex(file, sep=None, header="infer", skiprows=None, on_bad_lines='ski
     encoding_csv = get_encoding_of_file(file)
     to_file1 = os.path.join(os.path.dirname(file), f"{os.path.basename(file)[:-4]}_temp_修正.csv")
     if file[-3:].lower() == "txt" and sep is None:  # txt文件默认以空格作为分隔符
-        sep = "\s+"
+        sep = r"\s+"
     to_file1 = add_sep_to_csv(file, sep, to_file=to_file1, encoding=encoding_csv)
-    if file[-3:].lower() == "txt" and sep == "\s+":  # 如果该条件满足，则add_sep_to_csv已经将分隔符替换为"\t"
+    if file[-3:].lower() == "txt" and sep == r"\s+":  # 如果该条件满足，则add_sep_to_csv已经将分隔符替换为"\t"
         sep = "\t"
     if sep is None:
         sep = ","
