@@ -53,7 +53,7 @@ g_lunar_year_day = [
 from datetime import date, datetime, timedelta
 from yangke.web.webQA import *
 import re
-import sxtwl
+import sxtwl  # 该库已经停止更新，需要自己实现相关算法
 
 # 开始年份
 START_YEAR = 1901
