@@ -2,18 +2,16 @@
 win10系统专用，用于在windows操作系统中获取窗口、进程等应用程序操作
 """
 import ctypes
-import time
-
+import random
 import cv2
-import win32gui
+import win32gui  # pip install pypiwin32
 
-import win32con
 import win32process
 from PIL import Image
 import os
-import win32api
 from yangke.base import pic2ndarray, crop_pic
 from yangke.common.config import logger
+import numpy as np
 
 os.environ['KMP_DUPLICATE_LIB_OK'] = 'True'
 
@@ -26,8 +24,6 @@ def get_user32():
     """
     # user32.dll是stdcall
     from ctypes import windll
-    # import ctypes
-    # _user32 = ctypes.WinDLL("user32")
     return windll.user32
 
 
@@ -70,7 +66,9 @@ def get_all_window():
 
     def get_all_hwnd(hwnd, mouse):
         if win32gui.IsWindow(hwnd) and win32gui.IsWindowEnabled(hwnd) and win32gui.IsWindowVisible((hwnd)):
-            hwnd_title.update({hwnd: win32gui.GetWindowText(hwnd)})
+            title = win32gui.GetWindowText(hwnd)
+            if title != "":
+                hwnd_title.update({hwnd: win32gui.GetWindowText(hwnd)})
 
     win32gui.EnumWindows(get_all_hwnd, 0)
     return hwnd_title
@@ -85,7 +83,7 @@ def find_window(title, window_cls=None, exact=True):
     :param title: 查找的窗口名称
     :param window_cls: 窗口的类名，例如："GxWindowClass"、"Windows.UI.Core.CoreWindow"等，不指定则匹配所有类型的窗口
     :param exact: 是否精确查找，默认为True，为False是则匹配包含title的所有窗口
-    :return: exact为True是返回窗口的句柄，exact为False时返回{hwnd:title}
+    :return: exact为True是返回窗口的句柄，exact为False时返回{hwnd:title}字典
     """
     # 获取句柄
     window_cls = window_cls or 0
@@ -163,7 +161,7 @@ def get_pid_by_hwnd(hwnd):
     return thread_id, pid
 
 
-def get_size_of_window(hwnd, client=False):
+def get_size_of_window(hwnd, client=False) -> (int, int):
     """
     获取窗口大小
 
@@ -171,16 +169,65 @@ def get_size_of_window(hwnd, client=False):
     :param client: 是否获取客户区大小，即不包括标题框的窗口大小，默认为否，即返回总大小
     :return:
     """
-    if client:
-        left, top, right, bot = win32gui.GetClientRect(hwnd)
-    else:
-        left, top, right, bot = win32gui.GetWindowRect(hwnd)
+    try:
+        if client:
+            left, top, right, bot = win32gui.GetClientRect(hwnd)  # 内容区域大小，left和top永远为0
+        else:
+            left, top, right, bot = win32gui.GetWindowRect(hwnd)  # 该函数返回坐标
+    except Exception as e:
+        print(e)
+        return 0, 0, 0, 0
     width = right - left
     height = bot - top
     return width, height
 
 
-def capture_pic_undecorated(hwnd, x1=0, y1=0, x2=0, y2=0, save_to="temp.png"):
+def get_pos_of_window(hwnd, client=False) -> (int, int):
+    """
+    获取窗口位置
+    :param hwnd: 窗口句柄
+    :param client: 是否包含标题区域，True则返回窗口内容区域的位置，False则返回包括标题框的窗口位置
+    """
+    left, top, right, bottom = win32gui.GetWindowRect(hwnd)
+    if client:
+        w_, h_ = get_size_of_window(hwnd)  # 窗口总大小
+        w, h = get_size_of_window(hwnd, client=True)  # 客户去大小，即不包括标题框的区域的大小
+        pad_x = int((w_ - w) / 2)
+        pad_y_t = int(h_ - h - pad_x)
+        pad_y_b = int(h_ - h - pad_y_t)
+        return left + pad_x, top + pad_y_t
+    else:
+        return left, top
+
+
+def get_rect_of_window(hwnd, client=False) -> (int, int, int, int):
+    """
+    获取窗口的显示信息，返回 left, top, right, bottom
+    :param hwnd: 窗口句柄
+    :param client: 是否包含标题区域，True则返回窗口内容区域的位置，False则返回包括标题框的窗口位置
+    """
+    # if client:
+    #     w_, h_ = get_size_of_window(hwnd)  # 窗口总大小
+    #     w, h = get_size_of_window(hwnd, client=True)  # 客户去大小，即不包括标题框的区域的大小
+    #     pad_x = int((w_ - w) / 2)
+    #     pad_y_t = int(h_ - h - pad_x)
+    #     pad_y_b = int(h_ - h - pad_y_t)
+    # return left + pad_x, top + pad_y_t,
+    return win32gui.GetWindowRect(hwnd)
+
+
+def set_foreground(hwnd):
+    """
+    激活窗口到前台
+    """
+    try:
+        win32gui.SetForegroundWindow(hwnd)
+    except Exception as e:
+        logger.warning("激活窗口失败")
+        print(e)
+
+
+def capture_pic_undecorated(hwnd, x1=0, y1=0, x2=0, y2=0, save_to="memory"):
     """
     截取窗口不包括标题框的区域的画面
 
@@ -192,33 +239,39 @@ def capture_pic_undecorated(hwnd, x1=0, y1=0, x2=0, y2=0, save_to="temp.png"):
     :param save_to:
     :return:
     """
-    capture_pic(hwnd, save_to=save_to)
+    pic = capture_pic(hwnd, save_to=save_to)
     w_, h_ = get_size_of_window(hwnd)  # 窗口总大小
     w, h = get_size_of_window(hwnd, client=True)  # 客户去大小，即不包括标题框的区域的大小
     pad_x = int((w_ - w) / 2)
     pad_y_t = int(h_ - h - pad_x)
     pad_y_b = int(h_ - h - pad_y_t)
-    crop_pic(save_to, pad_x, pad_y_t, w_ - pad_x, h_ - pad_y_b, save_to=save_to)  # 裁掉图片中窗口的修饰框
+    pic, suc = crop_pic(pic, pad_x, pad_y_t, w_ - pad_x, h_ - pad_y_b, save_to=save_to)  # 裁掉图片中窗口的修饰框
+    if len(pic) == 0:
+        return []
     if (x1, y1, x2, y2) != (0, 0, 0, 0):  # 如果需要传入了截图区域，则对图片进行裁剪
-        crop_pic(save_to, x1, y1, x2, y2, save_to)  # 对窗口客户区中的区域进行裁剪
-    return save_to
+        pic, suc = crop_pic(pic, x1, y1, x2, y2, save_to)  # 对窗口客户区中的区域进行裁剪
+    return pic
 
 
-def capture_pic(hwnd, x1=0, y1=0, x2=0, y2=0, save_to: str = "temp.png", show=False):
+def capture_pic(hwnd, x1=0, y1=0, x2=0, y2=0, save_to: str = "memory", show=False):
     """
     截取给定句柄对应的窗口中指定位置的图像，可以处理后台画面，默认截取整个窗口画面。
-    测试游戏画面可以截取成功，如逆水寒，但Windows的记事本、计算器截取不成功
+    测试游戏画面可以截取成功，如逆水寒，但Windows的记事本、计算器截取不成功.
 
     :param hwnd: 要截图的窗口句柄
     :param x1: 截图区域的左上角x坐标，
     :param y1: 截图区域的左上角y坐标
     :param x2: 截图区域的右下角x坐标
     :param y2: 截图区域的右下角y坐标
-    :param save_to: 保存截图到文件时，通过该参数指定文件名
+    :param save_to: 保存截图到文件时，通过该参数指定文件名，不指定则为内存中保存的图片，否则，返回图片的ndarray数组，图片是RGB模式
     :return:
     """
     import win32ui
-    left, top, right, bot = win32gui.GetWindowRect(hwnd)
+    try:
+        left, top, right, bot = win32gui.GetWindowRect(hwnd)
+    except:
+        logger.error(f"无效的窗口句柄：{hwnd}")
+        return None
     width = right - left
     height = bot - top
     # 获取句柄窗口的设备环境，覆盖整个窗口，包括非客户区，标题栏，菜单，边框
@@ -243,11 +296,15 @@ def capture_pic(hwnd, x1=0, y1=0, x2=0, y2=0, save_to: str = "temp.png", show=Fa
     # 方法二：PIL保存
     bmp_info = saveBitMap.GetInfo()
     bmp_str = saveBitMap.GetBitmapBits(True)
-    im_PIL = Image.frombuffer('RGB', (bmp_info['bmWidth'], bmp_info['bmHeight']), bmp_str, 'raw', 'BGRX', 0, 1)
+    im_pil = Image.frombuffer('RGB', (bmp_info['bmWidth'], bmp_info['bmHeight']), bmp_str, 'raw', 'BGRX', 0, 1)
 
-    im_PIL.save(save_to)
+    if save_to == "memory":  # 如果需要的是内存对象
+        # noinspection all
+        save_to = np.asarray(im_pil)
+    else:
+        im_pil.save(save_to)
     if show:
-        im_PIL.show()
+        im_pil.show()
 
     # 内存释放
     win32gui.DeleteObject(saveBitMap.GetHandle())
@@ -256,6 +313,7 @@ def capture_pic(hwnd, x1=0, y1=0, x2=0, y2=0, save_to: str = "temp.png", show=Fa
     win32gui.ReleaseDC(hwnd, hwnd_DC)
     if (x1, y1, x2, y2) != (0, 0, 0, 0):  # 如果需要传入了截图区域，则对图片进行裁剪
         crop_pic(save_to, x1, y1, x2, y2, save_to)
+    return save_to
 
 
 def 注入dll(目标进程ID, dll_path):
@@ -271,11 +329,20 @@ def 注入dll(目标进程ID, dll_path):
 
 def find_pic(template: str, screen: str = "temp.png", threshold: float = 0.8):
     """
-    从图片中查找另一张图片的位置
+    从图片中查找另一张图片的位置，找到的坐标必然为正值，该方法对光照敏感，不同光照可能导致匹配错误；
 
-    :param screen: 大图片，即在该图片中查找
-    :param template: 小图片，即在大图片中查找该图片的相对位置
-    :param threshold: 阈值，越大则要求越相似
+    methods = {
+    cv2.TM_SQDIFF: "平方差匹配 - 最小值为最佳匹配",
+    cv2.TM_SQDIFF_NORMED: "归一化平方差匹配 - 对光照敏感",
+    cv2.TM_CCORR: "相关匹配 - 对亮度敏感",
+    cv2.TM_CCORR_NORMED: "归一化相关匹配 - 比TM_CCORR稍好",
+    cv2.TM_CCOEFF: "相关系数匹配 - 考虑整体统计特性",
+    cv2.TM_CCOEFF_NORMED: "归一化相关系数匹配 - 最佳选择，对光照变化鲁棒"
+}
+
+    :param screen: 大图片，即在该图片中查找，如果是数组，颜色通道顺序需要为RGB
+    :param template: 小图片，即在大图片中查找该图片的相对位置，如果是数组，颜色通道顺序需要为RGB
+    :param threshold: 阈值，越大则要求越相似，建议0.7以上，否则容易错误匹配
     :return: exists, (x,y)
     """
     threshold = 1 - threshold
@@ -307,11 +374,130 @@ def find_pic(template: str, screen: str = "temp.png", threshold: float = 0.8):
     return exist, coordinate
 
 
+def find_pic1(template: str, screen: str = "temp.png", threshold: float = 0.8):
+    """
+    从图片中查找另一张图片的位置，找到的坐标必然为正值
+
+    :param screen: 大图片，即在该图片中查找，如果是数组，颜色通道顺序需要为RGB
+    :param template: 小图片，即在大图片中查找该图片的相对位置，如果是数组，颜色通道顺序需要为RGB
+    :param threshold: 阈值，越大则要求越相似，建议0.7以上，否则容易错误匹配
+    :return: exists, (x,y), similarity
+    """
+    try:
+        scr = pic2ndarray(screen)
+        tp = pic2ndarray(template)
+        # 改用 TM_CCOEFF_NORMED 方法
+        result = cv2.matchTemplate(scr, tp, cv2.TM_CCOEFF_NORMED)
+    except cv2.error:
+        print('文件错误：', screen, template)
+        time.sleep(1)
+        try:
+            scr = cv2.imread(screen)
+            tp = cv2.imread(template)
+            result = cv2.matchTemplate(scr, tp, cv2.TM_CCOEFF_NORMED)
+        except cv2.error:
+            return False, None, 0.0
+
+    min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(result)
+
+    # 对于 TM_CCOEFF_NORMED，使用最大值作为相似度
+    similarity = max_val
+
+    # 总是返回最匹配的位置，但通过 exists 标记是否达到阈值
+    exist = similarity >= threshold
+
+    logger.debug(f"找图结果: 相似度={similarity:.2f}, 阈值={threshold}, 匹配={'是' if exist else '否'}")
+
+    if exist:
+        logger.debug(f"找图可信度较高:{os.path.basename(template)}")
+    else:
+        logger.warning(f"找图可信度较低:{os.path.basename(template)}")
+
+    # 对于 TM_CCOEFF_NORMED，使用最大值位置
+    coordinate = max_loc
+    return exist, coordinate, similarity
+
+
+def find_pic_sift(template, screen, threshold=0.7):
+    """
+    使用SIFT特征点匹配
+    """
+    scr = pic2ndarray(screen)
+    tp = pic2ndarray(template)
+
+    # 转换为灰度图
+    if len(scr.shape) == 3:
+        gray_scr = cv2.cvtColor(scr, cv2.COLOR_RGB2GRAY)
+    else:
+        gray_scr = scr
+
+    if len(tp.shape) == 3:
+        gray_tp = cv2.cvtColor(tp, cv2.COLOR_RGB2GRAY)
+    else:
+        gray_tp = tp
+
+    # 创建SIFT检测器，兼容不同版本的OpenCV
+    try:
+        # OpenCV 4.x及以上版本
+        sift = cv2.SIFT_create()
+    except AttributeError:
+        try:
+            # OpenCV 3.x版本
+            sift = cv2.xfeatures2d.SIFT_create()
+        except AttributeError:
+            # 更老的版本或其他情况
+            sift = cv2.Feature2D_create("SIFT")
+
+    # 检测关键点和描述符
+    kp1, des1 = sift.detectAndCompute(gray_tp, None)
+    kp2, des2 = sift.detectAndCompute(gray_scr, None)
+
+    if des1 is None or des2 is None:
+        return False, None, 0
+
+    # 特征点匹配
+    bf = cv2.BFMatcher()
+    matches = bf.knnMatch(des1, des2, k=2)
+
+    # 应用Lowe比率测试
+    good_matches = []
+    for m, n in matches:
+        if m.distance < threshold * n.distance:
+            good_matches.append(m)
+
+    # 至少需要4个匹配点来计算单应性矩阵
+    if len(good_matches) > 4:
+        # 提取匹配点坐标
+        src_pts = np.float32([kp1[m.queryIdx].pt for m in good_matches]).reshape(-1, 1, 2)
+        dst_pts = np.float32([kp2[m.trainIdx].pt for m in good_matches]).reshape(-1, 1, 2)
+
+        # 计算单应性矩阵
+        M, mask = cv2.findHomography(src_pts, dst_pts, cv2.RANSAC, 5.0)
+
+        if M is not None:
+            # 计算模板在大图中的位置
+            h, w = tp.shape[:2]
+            pts = np.float32([[0, 0], [0, h - 1], [w - 1, h - 1], [w - 1, 0]]).reshape(-1, 1, 2)
+            dst = cv2.perspectiveTransform(pts, M)
+
+            # 返回中心点坐标
+            center_x = int(np.mean([point[0][0] for point in dst]))
+            center_y = int(np.mean([point[0][1] for point in dst]))
+
+            # 计算匹配质量（基于匹配点数量）
+            similarity = len(good_matches) / max(len(kp1), len(kp2))
+
+            return True, (center_x, center_y), similarity
+
+    return False, None, 0
+
+
 def do_click(cx, cy, hwnd, right=False):
     """
     在窗口上点击鼠标，测试hwnd为桌面句柄时成功。
     逆水寒游戏窗口测试失败。
 
+
     :param right: 是否右键点击，默认否
     :param cx: 点击位置在窗口的相对x坐标
     :param cy: 点击位置在窗口的相对y坐标
@@ -325,75 +511,129 @@ def do_click(cx, cy, hwnd, right=False):
         time.sleep(0.1)
         win32api.PostMessage(hwnd, win32con.WM_RBUTTONUP, win32con.MK_RBUTTON, long_position)
     else:
-        win32api.PostMessage(hwnd, win32con.WM_LBUTTONDOWN, win32con.MK_LBUTTON, long_position)
+        # win32api.PostMessage(hwnd, win32con.WM_LBUTTONDOWN, win32con.MK_LBUTTON, long_position)
+        # time.sleep(0.1)
+        # win32api.PostMessage(hwnd, win32con.WM_LBUTTONUP, win32con.MK_LBUTTON, long_position)
+        win32api.SendMessage(hwnd, win32con.WM_LBUTTONDOWN, win32con.MK_LBUTTON, long_position)
         time.sleep(0.1)
-        win32api.PostMessage(hwnd, win32con.WM_LBUTTONUP, win32con.MK_LBUTTON, long_position)
+        win32api.SendMessage(hwnd, win32con.WM_LBUTTONUP, win32con.MK_LBUTTON, long_position)
 
 
 def key_press(hwnd, key):
     ...
 
 
-def post_key(hwnd, key, shift, specialkey=False, last_time=0.01):
+def post_key(hwnd, key, last_time=0.01):
     """
-    向指定窗口发送按键事件，记事本已测试成功，逆水寒测试成功。但部分按键的功能与实际按键功能不一致，原因未知。
+    向指定窗口发送按键事件。
+    记事本已测试成功，需要注意的是hwnd需要只想记事本的编辑区域的子窗口。
+    逆水寒测试成功。但部分按键的功能与实际按键功能不一致，原因未知。
+    天谕测试部分成功，单个按键功能正常，但是组合按键中的shift、Alt和Ctrl失效，原因未知。
     某些应用可能无法使用，测试失败的有：斗鱼客户端(客户端会激活，但功能无效)
 
     示例：
     hwnd = 记事本编辑框的窗口句柄 # 需要注意的是，如果是Ctrl+S这类功能按键，窗口句柄是记事本的根句柄还是下面的字句并都无所谓，但如果是编辑内容的输入，如输入A字母，则必须是编辑子窗口的句柄
-    PostKeyEx(hwnd, ord('A'), [win32con.VK_CONTROL], False)  # 全选记事本内容，相当于发送Ctrl+A按键事件
-    PostKeyEx(hwnd, ord('S'), [win32con.VK_MENU], False)  # 打开记事本设置面板，相当于发送Alt+S按键事件
-    PostKeyEx(hwnd, ord('A'), None, False)  # 在编辑区输入字母a，相当于在编辑区按“A"键
+    post_key(hwnd, 'Ctrl+A')  # 全选记事本内容，相当于发送Ctrl+A按键事件
+    post_key(hwnd, 'Ctrl+S')  # 打开记事本设置面板，相当于发送Alt+S按键事件
+    post_key(hwnd, 'A')  # 在编辑区输入字母a，相当于在编辑区按“A"键
+    post_key(hwnd, 'escape')  # 按Esc键
+
+    虚拟键码参加以下链接
+    https://learn.microsoft.com/zh-cn/windows/win32/inputdev/virtual-key-codes
+
     :param hwnd: 窗口句柄。
-    :param key: 需要点击的普通键，如ord('A')等
-    :param shift:
-    :param specialkey:
+    :param key: 需要点击的普通键，如'A', '1'等，具体参见keyboard.py中的VkCode的键值
     :param last_time: 按键持续时间，即键位按下的时间，单位为s
     :return:
     """
+    """
+    微软PostMessage函数原型为：
+    BOOL PostMessageW(HWND hwnd, UINT Msg, WPARAM wParam, LPARAM lParam)
+    其中hWnd是窗口句柄，Msg是消息的ID，wParam和lParam都需要按照消息的要求进行设置。每个消息的要求都不一样，具体如何设置，需要查看对应消息的文档。
+    """
     _user32 = get_user32()
     PBYTE256 = ctypes.c_ubyte * 256
-    if win32gui.IsWindow(hwnd):
-        ThreadId = _user32.GetWindowThreadProcessId(hwnd, None)  # win32process.GetWindowThreadProcessId给出了错误的值
+    time_last = last_time + random.randint(1, 10) / 100
 
-        lparam = win32api.MAKELONG(0, _user32.MapVirtualKeyA(key, 0))
+    special_key = False  # 不知道有什么用，Ctrl、Alt、Shift等按键都不需要设置该参数
+
+    # -------------------------- 将按键字符串转换为PostMessage可以识别的参数 -----------------------------------
+    addition_key = None  # 附加的功能按键
+    if '+' in key:
+        keys = key.split('+')
+        for k in keys:
+            if k.lower() == "ctrl":
+                if addition_key is None or addition_key == []:
+                    addition_key = [win32con.VK_CONTROL]
+                else:
+                    addition_key.append(win32con.VK_CONTROL)
+            elif k.lower() == 'shift':
+                if addition_key is None or addition_key == []:
+                    addition_key = [win32con.VK_SHIFT]
+                else:
+                    addition_key.append(win32con.VK_SHIFT)
+            elif k.lower() == 'alt':
+                if addition_key is None or addition_key == []:
+                    addition_key = [win32con.VK_MENU]
+                else:
+                    addition_key.append(win32con.VK_MENU)
+            else:
+                key = k
+    key = get_virtual_keycode(key)  # 虚拟键码等于字符的ASCII码值
+    # -------------------------- 将按键字符串转换为PostMessage可以识别的参数 -----------------------------------
+
+    if win32gui.IsWindow(hwnd):
+        thread_id = _user32.GetWindowThreadProcessId(hwnd, None)  # win32process.GetWindowThreadProcessId给出了错误的值
+
+        lparam = win32api.MAKELONG(0, _user32.MapVirtualKeyA(key, 0))  # 将虚拟按键code转换成长整形
+        # lparam_ctrl = win32api.MAKELONG(0, MapVirtualKeyW(win32con.VK_CONTROL, 0)) | 0x00000001
         msg_down = win32con.WM_KEYDOWN
         msg_up = win32con.WM_KEYUP
 
-        if specialkey:
+        if special_key:
             lparam = lparam | 0x1000000
 
         # 如果有修改-使用PostMessage和AttachThreadInput
         pKeyBuffers = PBYTE256()
         pKeyBuffers_old = PBYTE256()
 
-        win32gui.SendMessage(hwnd, win32con.WM_ACTIVATE, win32con.WA_ACTIVE, 0)
-        _user32.AttachThreadInput(win32api.GetCurrentThreadId(), ThreadId, True)
-        # 记录当前按下的组合键，如ctrl、shift、alt的状态，状态记录在pKeyBuffers_old中
-        _user32.GetKeyboardState(ctypes.byref(pKeyBuffers_old))
+        win32gui.SendMessage(hwnd, win32con.WM_ACTIVATE, win32con.WA_ACTIVE, 0)  # 激活hwnd窗口
 
         # ------------------------- 如果是组合按键，才需要设置键盘状态 --------------------------------
-        if shift is not None and len(shift) > 0:
+        if addition_key is not None and len(addition_key) > 0:
+            # 发送组合键时需要AttachThreadInput方法
+            _user32.AttachThreadInput(win32api.GetCurrentThreadId(), thread_id, True)  # 附加python线程至目标窗口
+            # 记录当前按下的组合键，如ctrl、shift、alt的状态，状态记录在pKeyBuffers_old中
+            _user32.GetKeyboardState(ctypes.byref(pKeyBuffers_old))
+
             # 记录需要按下的组合键，如ctrl、shift、alt的状态，状态记录在pKeyBuffers中
-            for mod_key in shift:
+            for mod_key in addition_key:
                 if mod_key == win32con.VK_MENU:  # Alt键需要单独处理，VK_MENU就是ALT键
                     lparam = lparam | 0x20000000
                     msg_down = win32con.WM_SYSKEYDOWN
                     msg_up = win32con.WM_SYSKEYUP
                 pKeyBuffers[mod_key] |= 128
             _user32.SetKeyboardState(ctypes.byref(pKeyBuffers))  # 将当前的键盘状态设置为记录的pKeyBuffers
-            time.sleep(0.1)
+            time.sleep(time_last)
         # win32api.PostThreadMessage(ThreadId, msg_down, key, lparam)
         win32api.PostMessage(hwnd, msg_down, key, lparam)
-        time.sleep(0.1)
+        time.sleep(time_last)
         # win32api.PostThreadMessage(ThreadId, msg_up, key, lparam | 0xC0000000)
         win32api.PostMessage(hwnd, msg_up, key, lparam | 0xC0000000)
-        time.sleep(0.1)
+        time.sleep(time_last)
+
+        # win32api.PostMessage(hwnd, win32con.WM_SYSKEYDOWN, win32con.VK_MENU, 0x20380001)
+        # win32api.PostMessage(hwnd, win32con.WM_SYSKEYDOWN, 0x56, 0x20200001)  # 0x56是V键
+        # win32api.PostMessage(hwnd, win32con.WM_SYSCHAR, 0x76, 0x20200001)
+        # win32api.PostMessage(hwnd, win32con.WM_SYSKEYUP, 0x56, 0xE0200001)
+        # win32api.PostMessage(hwnd, win32con.WM_KEYUP, win32con.VK_MENU, 0xC0380001)
+        # win32api.PostMessage(hwnd, win32con.WM_SYSKEYDOWN, 0x56, 1 << 29)  # 第29位为1表示alt键按下
+
         # ------------------------- 如果是组合按键，恢复设置键盘状态 --------------------------------
-        if shift is not None and len(shift) > 0:
+        if addition_key is not None and len(addition_key) > 0:
             _user32.SetKeyboardState(ctypes.byref(pKeyBuffers_old))
-            time.sleep(0.01)
-        _user32.AttachThreadInput(win32api.GetCurrentThreadId(), ThreadId, False)
+            time.sleep(0.02)
+            _user32.AttachThreadInput(win32api.GetCurrentThreadId(), thread_id, False)  # 从目标线程上分离python线程
 
 
 def send_message(hwnd, str):
@@ -429,10 +669,10 @@ if __name__ == "__main__":
     # post_key(hwnd, ord('F'), [win32con.VK_CONTROL])
     from yangke.common.win.keyboard import *
 
-    hwnd = find_window("记事本", exact=False)
-    hwnd = list(hwnd.keys())[0]
-    post_key(hwnd, ord("A"), [])
-    capture_pic(hwnd)
+    hWnd = find_window("记事本", exact=False)
+    hWnd = list(hWnd.keys())[0]
+    post_key(hWnd, ord("A"), [])
+    capture_pic(hWnd)
     while True:
-        click_key(hwnd, "V")
-# [722220, 394318]
+        click_key(hWnd, "V")
+    # [722220, 394318]
