@@ -70,8 +70,16 @@ def runCMD(command: str, charset: str = "utf8", wait_for_result: bool = True, cw
                 line, err = popen.communicate()  # command有大量输出时，popen.wait()可能会死锁
                 # 这里获得的lines是字节码格式，pycharm显示为 b'CONTAINER ID ...\n' ，前面的b指的是bytes
                 # 解码lines为字符串
-                line = line.decode(charset)
-                err = err.decode(charset)
+                try:
+                    line = line.decode(charset)
+                except UnicodeDecodeError:
+                    logger.debug("消息解码失败，尝试使用gbk编码重试")
+                    line = line.decode('gbk')
+                try:
+                    err = err.decode(charset)
+                except UnicodeDecodeError:
+                    logger.debug("错误信息解码失败，尝试使用gbk编码重试")
+                    err = err.decode('gbk')
                 return line, err
             else:
                 return popen.stdout, popen.stderr
