@@ -2,7 +2,7 @@
 数据分析软件
 """
 import pandas as pd
-from PyQt5 import QtCore
+from yangke.common.QtImporter import QtCore
 
 from yangke.performance.basic_para import Case, alternative_symbol
 from yangke.base import extend_list_by_alternative_symbols, get_temp_para, save_temp_para
@@ -11,9 +11,9 @@ from yangke.common.config import logger
 import copy
 from yangke.common.qt import YkWindow, YkDataTableWidget
 from yangke.performance.data_process import suffix_imp_20, suffix_p_20, suffix_dcs_20
-from PyQt5.QtWidgets import QApplication, QFileDialog, QProgressBar
-from PyQt5.QtGui import QBrush, QRadialGradient
-from PyQt5.QtCore import Qt, QObject, pyqtSignal
+from yangke.common.QtImporter import QApplication, QFileDialog, QProgressBar
+from yangke.common.QtImporter import QBrush, QRadialGradient
+from yangke.common.QtImporter import Qt, QObject, pyqtSignal
 import sys
 import os
 from yangke.performance.da import update_data, generate_case_skeleton
