@@ -10,7 +10,7 @@ import warnings  # 屏蔽MATPLOTLIBDATA警告
 warnings.filterwarnings("ignore", "(?s).*MATPLOTLIBDATA.*", category=UserWarning)
 
 from yangke.common.config import logger
-from yangke.common.fileOperate import write_func, read_func, writeAsPickle, readFromPickle, get_last_modified_time
+from yangke.common.fileOperate import write_func, read_func, write_as_pickle, read_from_pickle, get_last_modified_time
 from yangke.base import Line, interpolate_value_complex
 import os
 from yangke.base import get_settings, plot_3d, get_args, interpolate_2d
@@ -352,7 +352,7 @@ def read_fun_data():
         global coal_consume_2_func
         coal_consume_2_func = read_func("coal_consume_2_func.dat")
         global fl1_range_points
-        fl1_range_points = readFromPickle("fl_range_points.dat")
+        fl1_range_points = read_from_pickle("fl_range_points.dat")
         return True
     except:
         print("未找到函数文件power1_func.dat")
@@ -401,8 +401,7 @@ class MainWindow(YkWindow):
         self.splitter_r.addWidget(QLabel("最优个体随遗传代数变化"))
         self.tab.addTab(self.splitter_r, "图像")
         self.tab.addTab(self.table_widget, "数据")
-        self.splitter_l.addWidget(YkInputPanel(parent=self,
-                                               from_file=os.path.join(os.path.dirname(__file__),
+        self.splitter_l.addWidget(YkInputPanel(from_file=os.path.join(os.path.dirname(__file__),
                                                                       "ui/ui_data.yaml"),
                                                domain="calculate2"))
         self.result_panel = self.get_result_panel()
@@ -491,7 +490,8 @@ class MainWindow(YkWindow):
 
         self.table_widget.set_value("全厂（最优工况下）.总低压供汽量", round(self.fl_total, self.digits))
         self.table_widget.set_value("全厂（最优工况下）.全厂总负荷", round(p_total, self.digits))
-        self.table_widget.set_value("全厂（最优工况下）.全厂平均热耗", round((p1 * hr1 + p2 * hr2) / p_total, self.digits))
+        self.table_widget.set_value("全厂（最优工况下）.全厂平均热耗",
+                                    round((p1 * hr1 + p2 * hr2) / p_total, self.digits))
         self.table_widget.set_value("全厂（最优工况下）.全厂平均煤耗", round(coal_total_opt, self.digits))
         # 计算1号机单独供汽的工况
         fl1 = self.fl_total
@@ -512,7 +512,8 @@ class MainWindow(YkWindow):
                                         str(round(increase, 2)) + "%")
         self.table_widget.set_value("全厂（1号机单独供汽）.总低压供汽量", round(self.fl_total, self.digits))
         self.table_widget.set_value("全厂（1号机单独供汽）.全厂总负荷", round(p_total, self.digits))
-        self.table_widget.set_value("全厂（1号机单独供汽）.全厂平均热耗", round((p1 * hr1 + p2 * hr2) / p_total, self.digits))
+        self.table_widget.set_value("全厂（1号机单独供汽）.全厂平均热耗",
+                                    round((p1 * hr1 + p2 * hr2) / p_total, self.digits))
         self.table_widget.set_value("全厂（1号机单独供汽）.全厂平均煤耗", round(coal_total_if_only_1, self.digits))
 
         self.table_widget.set_value("#1低压供汽范围.下限", round(fl1_range[0], self.digits))
@@ -682,9 +683,9 @@ class MainWindow(YkWindow):
             global power1_func
             num_1_file = self.data_file or r"E:\2020\科研项目\应城电厂热电分配\data.xlsx"
             modified_time = get_last_modified_time(file=num_1_file)
-            modified_time_1 = readFromPickle("last_modified_time.dat")
+            modified_time_1 = read_from_pickle("last_modified_time.dat")
             if modified_time != modified_time_1:
-                writeAsPickle("last_modified_time.dat", modified_time)
+                write_as_pickle("last_modified_time.dat", modified_time)
             # ================================ 加载1号机功率数据==========================================
             logger.debug("加载1号机功率数据")
             # 默认用第一行作为列索引，使用index_col=0设置用第一列作为行索引
@@ -732,7 +733,7 @@ class MainWindow(YkWindow):
                 temp_series = data[:][fm].dropna().index
                 # 因为小数涉及到精度问题，当用其作为字典的键时，最好转换为字符串
                 fl1_range_points.update({str(fm): [temp_series.min(), temp_series.max()]})
-            writeAsPickle("fl_range_points.dat", fl1_range_points)
+            write_as_pickle("fl_range_points.dat", fl1_range_points)
             # ============================== 生成指定中压抽汽量下对应的低压抽汽量的取值范围=================================
 
             # =================================加载1号机煤耗数据===============================================
@@ -853,7 +854,8 @@ class MainWindow(YkWindow):
         self.statusBar1.showMessage("就绪")
 
     def set_algorithm(self):
-        input_panel = YkInputPanel(self, os.path.join(os.path.dirname(__file__), "ui/ui_data.yaml"), "algorithm")
+        input_panel = YkInputPanel(from_file=os.path.join(os.path.dirname(__file__), "ui/ui_data.yaml"),
+                                   domain="algorithm")
         if self.display is not None and len(self.display) == len(input_panel.findChildren(YkItem)):
             t = self.display
             values = [t.get("pop_size"), t.get("gens"), t.get("p_mutate"), t.get("p_cross"),
@@ -863,7 +865,8 @@ class MainWindow(YkWindow):
         self.statusBar1.showMessage("就绪")
 
     def set_display(self):
-        input_panel = YkInputPanel(self, os.path.join(os.path.dirname(__file__), "ui/ui_data.yaml"), "display")
+        input_panel = YkInputPanel(from_file=os.path.join(os.path.dirname(__file__), "ui/ui_data.yaml"),
+                                   domain="display")
         if self.display is not None and len(self.display) == len(input_panel.findChildren(YkItem)):
             t = self.display
             values = [t.get("need_pop_fig"), t.get("method1"), t.get("method2"), t.get("pop_fig_type"),
@@ -873,13 +876,14 @@ class MainWindow(YkWindow):
         self.statusBar1.showMessage("就绪")
 
     def set_calculate(self):
-        input_panel = YkInputPanel(self, os.path.join(os.path.dirname(__file__), "ui/ui_data.yaml"), "calculate")
+        input_panel = YkInputPanel(from_file=os.path.join(os.path.dirname(__file__), "ui/ui_data.yaml"),
+                                   domain="calculate")
         input_panel.set_values([self.fm1, self.fl_total, self.p2])
         self.splitter_l.replaceWidget(0, input_panel)
         self.statusBar1.showMessage("就绪")
 
     def set_calculate2(self):
-        input_panel = YkInputPanel(self, os.path.join(os.path.dirname(__file__), "ui/ui_data.yaml"), "calculate2")
+        input_panel = YkInputPanel(from_file=os.path.join(os.path.dirname(__file__), "ui/ui_data.yaml"), domain="calculate2")
         input_panel.set_values([self.fm1, self.fl_total, self.p_total])
         self.splitter_l.replaceWidget(0, input_panel)
         self.statusBar1.showMessage("就绪")
@@ -1041,7 +1045,7 @@ if __name__ == "__main__":
     else:
         fm1 = float(args.fm1)
         fl_total = float(args.fl_total)
-        data_file = args.data_file
+        data_file = args.exe_file
         if type_cal == "1":
             p2 = float(args.p2)
             assert p2 and fm1 and fl_total
