@@ -73,7 +73,7 @@ def get_p_by_th(t, h):
     t_K = t + 273.15
     # 因为IAPWS97不能根据 焓 和 温度 求 压力，这里借助IAPWS95进行查找求解
     # 即多次计算 get_h_by_pt，对比计算的焓值与给定焓值相等则返回对应的p
-    p = IAPWS95(h=h, T=t_K).P
+    p = IAPWS95(h=h, T=t_K).P  # 该函数有时候结果很离谱，如焓值为763.9，温度为177.4℃时
     h_1 = get_h_by_pt(p=p, t=t)
     if h_1 > h:  # 温度不变时，压力越大，焓越低，考虑绝热管道的蒸汽流动，等焓，pt都降低
         # 说明估计的p偏小
@@ -248,5 +248,6 @@ if __name__ == "__main__":
     #         water = IAPWS97(P=p, T=t + 273.15)
     #         print(str(water.w) + ",", end="")
     #     print("")
-    water = IAPWS97(P=8.7, T=218 + 273.15)
-    print(str(water.w) + ",", end="")
+    # water = IAPWS97(H=763.9, T=117.4 + 273.15)
+    # print(str(water.P) + ",", end="")
+    print(P_TH(177.4, 763.9))
\ No newline at end of file
