@@ -17,7 +17,8 @@ from yangke.common.config import logger
 from yangke.common.fileOperate import read_csv_ex
 import os
 
-os.environ["KMP_DUPLICATE_LIB_OK"] = "TRUE"
+# 当Python程序中同时加载了多个包含OpenMP运行时库的模块（如NumPy、SciPy、PyTorch等）时，可能会出现库冲突
+os.environ["KMP_DUPLICATE_LIB_OK"] = "TRUE"  # 解决OpenMP库冲突问题，允许存在重复的OpenMP库而不报错
 
 
 class PriorLoss(nn.Module):
@@ -25,13 +26,13 @@ class PriorLoss(nn.Module):
     自定义的损失函数，可以使用已有的先验知识
     """
 
-    def __init__(self, input_titles, net):
+    def __init__(self, input_titles, net, punish=2000):
         self.input_titles = input_titles
         super(PriorLoss, self).__init__()
         self.rules = []
         self.reg_rules = []  # [("der>0, '电功率'")]  # 表示输出对电功率的导数大于0
         self.net = net  # 损失函数所属的神经网络
-        self.punish = 1000  # 当预测结果违背先验知识时的损失函数惩罚因子
+        self.punish = punish  # 当预测结果违背先验知识时的损失函数惩罚因子
 
     def forward(self, pred, target, input):
         """
@@ -42,16 +43,21 @@ class PriorLoss(nn.Module):
         :param input: 输入参数
         :return:  2866343884064  2866343893200
         """
-        x = input
-        x.requires_grad = True
-        if x.grad is not None:
-            x.grad = x.grad - x.grad  # 清空以前的梯度
+        # 创建新的可求导张量，避免影响原始输入
+        x = input.clone().detach().requires_grad_(True)
         predict = self.net(x)
-        # y对x求导
-        _y = predict.mean()
-        _y.backward()  # 只是计算梯度，不进行权值更新
-        grad = x.grad  # 则此处x.grad就是y对x的导数
-        x.requires_grad = False
+        # 对每个输出分别计算梯度
+        grads = []
+        for i in range(predict.shape[1]):  # 遍历所有输出
+            if predict.grad_fn is not None:
+                predict[:, i].sum().backward(retain_graph=True)
+                grads.append(x.grad.clone())
+                x.grad.zero_()  # 清零梯度，为下次计算准备
+        # 合并所有输出的梯度
+        if len(grads) > 0:
+            grad = torch.stack(grads, dim=-1).mean(dim=-1)  # 取平均作为最终梯度
+        else:
+            grad = torch.zeros_like(x)
 
         correction = self.apply_rules(x, grad)  # 先验知识的修正项
 
@@ -70,8 +76,8 @@ class PriorLoss(nn.Module):
         correction = torch.zeros_like(grad[:, 0])  # 无论输入有多少个，损失函数值只有一个，因此只取grad的第一个为维度
         for rr in self.reg_rules:
             if rr[0] == "der>0":
-                idx = self.input_titles.index(rr[1])
-                _cor = grad[:, idx]  # 所有输入对x的导数，该数应该大于0才符合先验知识
+                idx = self.input_titles.index(rr[1])  # 获取指定输入参数的索引
+                _cor = grad[:, idx]  # 获取该参数对应的梯度
                 _cor = -_cor * self.punish  # 如果导数小于0，则correction为大于0的数，即损失增大
                 _cor = torch.clamp(_cor, 0, None)  # 将所有小于0的数用0代替，即导数大于0时，损失修正值为0
                 correction = correction + _cor
@@ -98,7 +104,7 @@ class PriorLoss(nn.Module):
                 else:
                     logger.error(f"暂不支持带条件的先验知识")
                     expression = None
-            res = re.findall('\[(.+?)\]', expression)  # 加一个?则是最小匹配，否则时贪婪匹配
+            res = re.findall(r'\[(.+?)\]', expression)  # 加一个?则是最小匹配，否则时贪婪匹配
             plus = re.findall('.*单调递(.+)', expression)
             if len(res) == 2 and len(plus) == 1:
                 if plus[0] == "增":
@@ -139,6 +145,22 @@ class PriorLoss(nn.Module):
     def clear_knowledge(self):
         self.rules = []
 
+    def set_punish_factor(self, new_punish):
+        """
+        动态设置惩罚因子
+
+        :param new_punish: 新的惩罚因子值
+        """
+        self.punish = new_punish
+
+    def get_punish_factor(self):
+        """
+        获取当前惩罚因子值
+
+        :return: 当前惩罚因子值
+        """
+        return self.punish
+
 
 def standard_reverse(out, mean, std):
     """
@@ -174,7 +196,7 @@ def all_number(row: pd.Series):
 
 class DataSetFitting(Dataset):
     def __init__(self, data_source, x_title=None, y_title=None, mean=None, std=None, proportion_train=0.8,
-                 proportion_test=0.2):
+                 proportion_test=0.2, classify=False):
         """
         构建数据集。
         用法示例：
@@ -190,6 +212,7 @@ class DataSetFitting(Dataset):
         :param std: 数据集的标准差
         :param proportion_train: 训练集的比例，默认80%
         :param proportion_test: 测试集比例，默认20%
+        :param classify: 是否为分类问题的数据集，如果是分类问题，则归一化不会对标签应用，否则会将标签也归一化处理
         """
         self.dataframe = None
         self.x_title = None
@@ -236,6 +259,7 @@ class DataSetFitting(Dataset):
             # 删除某些行
             self.drop_by_condition(data_source.get_settings("dataset.drop"))
 
+        self.classify = classify
         # 仅保留x,y相关的数据列，删除其他列
         titles = self.x_title.copy()
         titles.extend(self.y_title)
@@ -280,6 +304,8 @@ class DataSetFitting(Dataset):
                     self.mean = self.mean or self.dataframe.mean()
                     self.std_range = self.std_range or self.dataframe.std()
                     self.dataframe_std = self.standard(self.dataframe)  # 必须放在这里，不然可能由于编译优化被忽略
+                    if self.classify:  # 分类问题，不能对标签进行归一化处理
+                        self.dataframe_std[self.y_title] = self.dataframe[self.y_title]
                     return
                 else:
                     self.x_range = x_range or self.x_range
@@ -311,6 +337,8 @@ class DataSetFitting(Dataset):
             self.normal_method = normal_method
             self.set_standard(mean=mean, std_range=std_range, normal_method=normal_method, x_range=x_range,
                               y_range=y_range)
+        if self.classify:  # 分类问题，不能对标签进行归一化处理
+            self.dataframe_std[self.y_title] = self.dataframe[self.y_title]
 
     def standard(self, df):
         """
@@ -349,7 +377,13 @@ class DataSetFitting(Dataset):
             self.dataframe_std = self.dataframe
         single_item = self.dataframe_std.iloc[index, :]
         _x = torch.from_numpy(single_item[self.x_title].to_numpy()).to(torch.float32)
-        _y = torch.from_numpy(single_item[self.y_title].to_numpy()).to(torch.float32)
+        if self.classify:  # 如果是分类问题，则输出必须是torch.LongTensor类型
+            # crossEntropyLoss会将整数转换成one-hot类型的数组，因此每一个标签必须是整数，不能是数组
+            _y = torch.from_numpy(single_item[self.y_title].to_numpy()).to(torch.long).squeeze()
+            # _y = _y.long()
+        else:
+            _y = torch.from_numpy(single_item[self.y_title].to_numpy()).to(torch.float32)
+
         return _x, _y
 
     def __len__(self):
@@ -427,7 +461,7 @@ class DataFitterNet(torch.nn.Module):
         :param trial: 使用optuna超参数调优时会传入该参数
         """
         super(DataFitterNet, self).__init__()
-        self.dataset: DataSetFitting = None  # 数据集的信息，主要需要使用数据集中的归一化与反归一化的信息，但不会保存dataframe数据
+        self.dataset: DataSetFitting | None = None  # 数据集的信息，主要需要使用数据集中的归一化与反归一化的信息，但不会保存dataframe数据
         cfg = settings1.get("networks")  # 获取神经网络结构信息
         settings1 = YkDict(settings1)
         self.settings = settings1
@@ -435,6 +469,13 @@ class DataFitterNet(torch.nn.Module):
         self.cfg = cfg
         self.in_num = 0  # 输入层神经元个数，对应输入参数的个数
         self.out_num = 1
+        loss_func_type = settings1.get_settings("networks.loss_fn.type")
+        if loss_func_type == "CrossEntropy":  # 必须是分类问题才能使用CrossEntropy损失函数
+            self.classify = True
+            self.class_num = settings1.get_settings("networks.loss_fn.bak_info")
+        else:
+            self.classify = False
+
         train_settings = settings1.get_settings("networks.train") or {}
         self.lr = self.get_para_of_optuna_str(train_settings.get("learning_rate") or 1e-3, "lr")
         self.epochs = int(train_settings.get("epochs") or 10)
@@ -490,15 +531,18 @@ class DataFitterNet(torch.nn.Module):
             elif _type.lower() == "none":  # 空层
                 continue
             elif _type == "output":  # 输入/输出层的神经元个数可以根据输入/输出参数个数自动确定
-                if layer.get("cell_num") == "auto" or layer.get("cell_num") is None:
-                    self.out_num = len(self.y_title)
-                elif settings1.get_settings("output.type") == "single_output":
-                    if layer.get("cell_num") != "auto" and int(layer.get("cell_num")) != 1:
-                        logger.warning(
-                            "数据集使用single_output，但神经网络模型输出层单元个数不为1，忽略神经网络输出单元个数设置！")
-                    self.out_num = 1
+                if self.classify:  # 如果是分类问题，则输出个数为类别数量，分别对应每一类的概率
+                    self.out_num = self.class_num
                 else:
-                    self.out_num = layer.get("cell_num")
+                    if layer.get("cell_num") == "auto" or layer.get("cell_num") is None:
+                        self.out_num = len(self.y_title)
+                    elif settings1.get_settings("output.type") == "single_output":
+                        if layer.get("cell_num") != "auto" and int(layer.get("cell_num")) != 1:
+                            logger.warning(
+                                "数据集使用single_output，但神经网络模型输出层单元个数不为1，忽略神经网络输出单元个数设置！")
+                        self.out_num = 1
+                    else:
+                        self.out_num = layer.get("cell_num")
                 bias = False if layer.get("bias") is False else True
                 layer_dict[f"layer{i}"] = nn.Linear(_cell_num_last_layer, self.out_num, bias=bias)
             else:
@@ -507,6 +551,9 @@ class DataFitterNet(torch.nn.Module):
             _cell_num_last_layer = _cell_num
             i = i + 1
 
+        if self.classify:
+            layer_dict[f"layer{i}"] = nn.Softmax(dim=1)  # nn.functional.softmax()
+
         self.net = nn.Sequential(layer_dict)
         weight_decay = self.get_para_of_optuna_str(settings1.get_settings("networks.optimizer.weight_decay") or 0,
                                                    "weight_decay")
@@ -522,7 +569,13 @@ class DataFitterNet(torch.nn.Module):
             self.loss_fn = MAXLoss()
         elif settings1.get_settings("networks.loss_fn.type") == "PriorLoss":
             self.loss_fn = PriorLoss(self.x_title, self.net)
-            self.loss_fn.add_knowledge(settings1.get_settings("networks.loss_fn.bak_info"))
+            self.loss_fn.add_knowledge([
+                {'isChecked': True, 'op': '任何条件', 'expression': '[背压]随[汽轮发电机功率]单调递增'},
+                {'isChecked': True, 'op': '任何条件', 'expression': '[背压]随[凝汽器热负荷]单调递增'},
+                {'isChecked': True, 'op': '任何条件', 'expression': '[背压]随[环境温度]单调递增'},
+                {'isChecked': True, 'op': '任何条件', 'expression': '[背压]随[风机运行台数]单调递减'}
+            ])
+            # self.loss_fn.add_knowledge(settings1.get_settings("networks.loss_fn.bak_info"))
         elif settings1.get_settings("networks.loss_fn.type") == "CrossEntropy":
             self.loss_fn = nn.CrossEntropyLoss()
         elif settings1.get_settings("networks.loss_fn.type") == "BCE":
@@ -536,13 +589,16 @@ class DataFitterNet(torch.nn.Module):
         out = self.net(x)
         return out
 
-    def prediction(self, x_input):
+    def prediction(self, x_input, is_normalized=False):
         """
         预测指定输入参数，输入参数应为未进行归一化处理的原始数据
 
         :param x_input: x.shape = torch.Size([batch_size, input_dim])，也可以是长度为input_dim的列表或ndarray
+        :param is_normalized: x_input是否已经归一化，默认为False，则prediction会执行归一化，否则不执行归一化直接预测
         :return:
         """
+        mean = self.mean
+        std = self.std
         if isinstance(self.mean, pd.Series):
             mean = self.mean[self.x_title].values
         if isinstance(self.std, pd.Series):
@@ -553,15 +609,20 @@ class DataFitterNet(torch.nn.Module):
         elif isinstance(x_input, list):
             x_input = np.asarray(x_input)
 
+        # 将x_input转换成torch.tensor类型
         if isinstance(x_input, np.ndarray):
-            x_input = (x_input - mean) / std  # 输入数据归一化
+            if not is_normalized:
+                x_input = (x_input - mean) / std  # 输入数据归一化
             x_input = torch.from_numpy(x_input)
         x_input = x_input.to(torch.float32)
         self.eval()
-        y = self.net(x_input)
-        res = standard_reverse(y, mean=self.mean[self.y_title], std=self.std[self.y_title])
-        if isinstance(res, np.ndarray) and res.size == 1:
-            res = res.item()
+        y = self.forward(x_input)
+        if self.classify:  # 分类问题，需要找到输出中概率最大的数的索引为最终的类别
+            res = torch.argmax(y, dim=1)
+        else:
+            res = standard_reverse(y, mean=self.mean[self.y_title], std=self.std[self.y_title])
+            if isinstance(res, np.ndarray) and res.size == 1:
+                res = res.item()
         return res
 
     def set_standard(self, mean, std, x_titles, y_titles, normal_method):
@@ -584,8 +645,9 @@ class DataFitterNet(torch.nn.Module):
     def save_yk(self, path):
         checkpoint = {
             'normal_method': self.normal_method,
-            'mean': self.mean,
-            'std': self.std,
+            # pandas.Series 对象在反序列化时可能执行任意代码，因此被 PyTorch 标记为不安全
+            'mean': self.mean.to_dict() if isinstance(self.mean, pd.Series) else self.mean,
+            'std': self.std.to_dict() if isinstance(self.std, pd.Series) else self.std,
             'x_title': self.x_title,
             'y_title': self.y_title,
             'model_state_dict': self.state_dict(),
@@ -601,9 +663,11 @@ class DataFitterNet(torch.nn.Module):
         if path is None or not os.path.exists(path):
             logger.debug("神经网络模型文件不存在！")
             return None
-        checkpoint = torch.load(path)
+        checkpoint = torch.load(path, weights_only=False)
         model1 = DataFitterNet(checkpoint.get("settings"))
-        model1.set_standard(checkpoint['mean'], checkpoint['std'], checkpoint['x_title'], checkpoint['y_title'],
+        model1.set_standard(pd.Series(checkpoint['mean']),
+                            pd.Series(checkpoint['std']),
+                            checkpoint['x_title'], checkpoint['y_title'],
                             checkpoint['normal_method'])
         model1.max_err = checkpoint['max_err']
         model1.average_err = checkpoint['average_err']
@@ -629,13 +693,13 @@ class DataFitterNet(torch.nn.Module):
             pb = tqdm(data_loader)
         else:
             pb = data_loader
-            pb.set_description = print
-        relative_err = 0
+            pb.set_description = logger.info
+        relative_err = 0  # 如果是分类问题，则记录的是分类错误的数据条目数量
         max_err = 0
         for batch, (x, y) in enumerate(pb):  # 这里的(x, y)是归一化后的数据
             x = x.to(self.device())
             y = y.to(self.device())
-            predict = self.net(x)
+            predict = self.forward(x)
 
             if isinstance(self.loss_fn, PriorLoss):  # 先验知识的损失函数需要y对x的导数
                 loss = self.loss_fn(predict, y, x)
@@ -658,12 +722,20 @@ class DataFitterNet(torch.nn.Module):
                 loss, current = loss.item(), batch * len(x)  # loss是Tensor对象，没有parent，使用tensor.item()取值
                 pb.set_description(f"loss: {loss:>7f}  [{current:>6d}/{size:>6d}-{self.y_title}]")
 
-            pred = self.dataset.standard_reverse(predict)  # 预测的真实值，任何情况都可以计算
-            _y = self.dataset.standard_reverse(y)  # 实际的真实值
-            max_err = max(max_err, (abs(pred - _y) / abs(_y)).max())  #
-            relative_err = ((pred - _y).abs() / _y.abs()).sum() + relative_err  # 所有批次总的相对误差
-        relative_err = relative_err / size  # 平均相对误差
-        logger.debug(f"训练集预测结果：相对误差为{relative_err.item():>8f}，最大误差为{max_err.item():>8f}")
+            if self.classify:
+                relative_err += torch.sum(y.flatten() != torch.argmax(predict, dim=1))
+            else:
+                # 累加当前的平均误差，更新当前的最大误差
+                pred = self.dataset.standard_reverse(predict)  # 预测的真实值，任何情况都可以计算
+                _y = self.dataset.standard_reverse(y)  # 实际的真实值
+                max_err = max(max_err, (abs(pred - _y) / abs(_y)).max())  #
+                relative_err = ((pred - _y).abs() / _y.abs()).sum() + relative_err  # 所有批次总的相对误差
+        if self.classify:
+            relative_err = relative_err / size
+            logger.debug(f"训练集测试结果：分类错误概率为{relative_err}")
+        else:
+            relative_err = relative_err / size  # 平均相对误差
+            logger.debug(f"训练集预测结果：相对误差为{relative_err.item():>8f}，最大误差为{max_err.item():>8f}")
         return loss, (relative_err, max_err)
 
     def test_loop(self, data_loader):
@@ -671,40 +743,51 @@ class DataFitterNet(torch.nn.Module):
         size = len(data_loader.dataset)
         test_loss = 0
         self.eval()  # 停止更新权值参数，dropout层全部激活
-        relative_err = 0
+        relative_err = 0  # 分类问题，则为分类错误的数据条目的个数
         max_err = 0
-        with torch.no_grad():  # 停止autograd模块的工作，不影响dropout和batch norm层的行为，可以和self.eval()同时使用
-            for _x, _y in data_loader:
-                _x = _x.to(self.device())
-                _y = _y.to(self.device())
-                pred = self.net(_x)
+        # 不要对整个循环使用torch.no_grad()，因为PriorLoss需要梯度计算
+        for _x, _y in data_loader:
+            _x = _x.to(self.device())
+            _y = _y.to(self.device())
+            pred = self.forward(_x)
 
-                # 不同的损失函数，需要传入的参数不一样
-                if isinstance(self.loss_fn, PriorLoss):  # 先验知识的损失函数需要y对x的导数
-                    # PriorLoss不支持torch.no_grad下计算损失
-                    # loss = self.loss_fn(pred, _y, _x)
-                    ...
-                elif isinstance(self.loss_fn, nn.MSELoss):
+            # 不同的损失函数，需要传入的参数不一样
+            if isinstance(self.loss_fn, PriorLoss):  # 先验知识的损失函数需要y对x的导数
+                # 为PriorLoss创建一个有梯度的输入副本
+                _x_grad = _x.clone().detach().requires_grad_(True)
+                pred_grad = self.forward(_x_grad)
+                loss = self.loss_fn(pred_grad, _y, _x_grad)
+                test_loss += loss.item()  # 所有批次总的损失函数值
+            elif isinstance(self.loss_fn, nn.MSELoss):
+                with torch.no_grad():  # 对于其他损失函数，可以使用torch.no_grad()
                     loss = self.loss_fn(pred, _y)
-                    test_loss += loss.item()  # 所有批次总的损失函数值
-                elif isinstance(self.loss_fn, nn.CrossEntropyLoss):
+                test_loss += loss.item()  # 所有批次总的损失函数值
+            elif isinstance(self.loss_fn, nn.CrossEntropyLoss):
+                with torch.no_grad():  # 对于其他损失函数，可以使用torch.no_grad()
                     loss = self.loss_fn(pred, _y)
-                    test_loss += loss.item()  # 所有批次总的损失函数值
-                elif isinstance(self.loss_fn, nn.CTCLoss):
-                    logger.error(f"暂不支持CTC损失函数")
-                    return
-                else:
-                    logger.error(f"暂不支持{self.loss_fn.__class__.__name__}损失函数")
+                test_loss += loss.item()  # 所有批次总的损失函数值
+            elif isinstance(self.loss_fn, nn.CTCLoss):
+                logger.error(f"暂不支持CTC损失函数")
+                return
+            else:
+                logger.error(f"暂不支持{self.loss_fn.__class__.__name__}损失函数")
 
-                pred = self.dataset.standard_reverse(pred)  # 预测的真实值
-                _y = self.dataset.standard_reverse(_y)  # 实际的真实值
-                # max_err = max(max_err, (abs(pred-y)/abs(y)).max(0))  # tensor.max()返回最大值,tensor.max(0)返回最大值和索引
-                max_err = max(max_err, (abs(pred - _y) / abs(_y)).max())  #
-                relative_err = ((pred - _y).abs() / _y.abs()).sum() + relative_err  # 所有批次总的相对误差
+            if self.classify:
+                with torch.no_grad():
+                    relative_err += torch.sum(_y.flatten() != torch.argmax(pred, dim=1))
+            else:
+                with torch.no_grad():
+                    pred_val = self.dataset.standard_reverse(pred)  # 预测的真实值
+                    _y_val = self.dataset.standard_reverse(_y)  # 实际的真实值
+                    # max_err = max(max_err, (abs(pred-y)/abs(y)).max(0))  # tensor.max()返回最大值,tensor.max(0)返回最大值和索引
+                    max_err = max(max_err, (abs(pred_val - _y_val) / abs(_y_val)).max())  #
+                    relative_err = ((pred_val - _y_val).abs() / _y_val.abs()).sum() + relative_err  # 所有批次总的相对误差
 
         relative_err = relative_err / size  # 平均相对误差
         if isinstance(self.loss_fn, PriorLoss):
             logger.debug(f"测试集预测结果({self.y_title[0]})")
+        elif self.classify:
+            logger.debug(f"测试集预测结果：分类错误的概率为：{relative_err}")
         else:
             test_loss /= num_batches  # 平均损失
             logger.debug(f"测试集预测结果({self.y_title[0]}): 平均损失为{test_loss:>8f}")
@@ -757,11 +840,15 @@ class DataFitterNet(torch.nn.Module):
             train_loss, train_err = self.train_loop(train_loader)
             test_loss, (test_relative_err, test_max_err) = self.test_loop(test_loader)
             relative_err.append(test_relative_err)
-            logger.debug(f"平均相对误差为：{test_relative_err:>8f}，最大相对误差为{test_max_err.item():>8f}")
+            if self.classify:
+                logger.debug(f"交叉熵损失为：{test_loss:>8f}，分类错误的概率为：{test_relative_err:>8f}")
+            else:
+                logger.debug(f"平均相对误差为：{test_relative_err:>8f}，最大相对误差为{test_max_err.item():>8f}")
             # ------------------------- 损失曲线图 ---------------------------------
             writer.add_scalar('train/损失', train_loss, t)
             writer.add_scalar('test/平均误差', test_relative_err, t)
-            writer.add_scalar('test/最大误差', test_max_err, t)
+            if not self.classify:
+                writer.add_scalar('test/最大误差', test_max_err, t)
             # ------------------------- 损失曲线图 ---------------------------------
             # ------------------------- 权重直方图 ---------------------------------
             for i, (name, param) in enumerate(self.named_parameters()):
