@@ -31,7 +31,7 @@ class MainWindow(YkWindow):
             self.project = Project(self.proj)
         else:
             self.project = Project()
-        self.enable_input_panel()
+        # self.enable_input_panel()
         self.symbols = self.get_stock_symbols()
 
         # ---------------------------- 初始化股票列表页 ------------------------------
