@@ -8,7 +8,6 @@ import sys
 import yangke.stock.dataset.data_reader as dr
 import pandas as pd
 from torch.utils.data import DataLoader, TensorDataset, Dataset
-from yangke.common.config import holdLoggingLevel
 import logging
 
 torch.manual_seed(10000)
@@ -60,12 +59,10 @@ def prediction(stock_codes: list, method: str = "fit") -> pd.DataFrame:
     for i in range(len(stock_codes)):
         stock_code = stock_codes[i]
         model_saved_path = os.path.join(model_saved_folder, "{}.pth".format(stock_code))
-        holdLoggingLevel(logging.WARNING)
         if '-1' in method:  # 如果是共用model进行预测，则传入公共model名
             pred = method1(stock_code, os.path.join(model_saved_folder, 'allInOneModel.pth'))
         else:
             pred = method1(stock_code, model_saved_path)
-        holdLoggingLevel()
         gv.logger.debug("预测进度(完成预测的股票数)/(总股票数)：{}/{}，当前预测股票代码：{}".format(i, len(stock_codes), stock_code))
         if pred is not None:  # 如果有预测结果，则添加，有些没有结果的则不添加
             stock_codes_.append(stock_code)
