@@ -1,6 +1,7 @@
+import json5
 from flask import Flask, request, jsonify, make_response, send_from_directory, Response
 from yangke.base import name_transfer
-from flask_cors import *
+from flask_cors import CORS, cross_origin
 from yangke.common.config import logger
 import traceback
 import os
@@ -11,7 +12,7 @@ example = None
 def start_server_app(deal=None, app_name='yangke.web.flaskserver', instance_path=__file__,
                      instance_relative_config=True, host="0.0.0.0", port="5000", allow_action=None, use_action=True,
                      example_url=None, content="json", template_folder="templates", static_folder="static",
-                     run_immediate=True):
+                     run_immediate=True, single_thread=False, ssl=False, cors=True):
     """
     启动Flask Server服务，服务启动后将调用用户传入的deal函数来对url请求进行处理，deal函数通过args参数拿到请求的参数字典。
     如果用户需要自定义deal函数处理http://127.0.0.1:5000这类没有参数的url，则应设置useAction为False，并在deal函数中对空字典进行判断处理。
@@ -30,14 +31,15 @@ def start_server_app(deal=None, app_name='yangke.web.flaskserver', instance_path
     start_server_app(deal, use_action=True)  # 启动服务器.
     #==============================================================================================
 
-    使用示例2：创建app后，可以通过@app.route('/')设置路由，主要是deal=None和run_immediate=False两个参数设置,通过run()方法启动服务
+    使用示例2：创建app后，可以通过@app.route('/')设置路由，主要是deal=None和run_immediate=False两个参数设置,通过run()方法启动服务。
+    该方法定义的路由函数不能从外部传入参数，必须通过request读取参数
     #==============================================================================================
     app = start_server_app(deal=None, use_action=False, template_folder=os.path.abspath("templates"), content="html",
                        run_immediate=False)
     @app.route("/add")
-    def add(args):
-        var1 = args.get("var1")
-        var2 = args.get("var2")
+    def add():
+        var1 = request.get("var1")
+        var2 = request.get("var2")
         return str(var1+var2)
     run(app)
     #==============================================================================================
@@ -55,13 +57,18 @@ def start_server_app(deal=None, app_name='yangke.web.flaskserver', instance_path
     :param template_folder: 静态资源路径
     :param static_folder: 除了原Flask的路径功能外，下载文件时默认的检索路径也是该目录
     :param run_immediate: 是否立即运行，默认为True,否则返回app对象，可以由用户进一步处理后，调用run(app)运行
+    :param single_thread: 是否需要单线程运行，默认False，有些com接口不能多线程运行
     :return:
     """
     app = Flask(app_name, instance_path=instance_path, instance_relative_config=instance_relative_config,
                 template_folder=template_folder, static_folder=static_folder)
     app.logger = logger
+    # if ssl:
+    #     from flask_sslify import SSLify
+    #     sslify = SSLify(app)
     app.config['JSON_AS_ASCII'] = False  # 让Flask返回的字符串可以正确显示中文，默认显示的是utf-8编码
-    CORS(app)
+    if cors:
+        CORS(app)
     global example
     example = example_url
 
@@ -78,9 +85,16 @@ def start_server_app(deal=None, app_name='yangke.web.flaskserver', instance_path
         args = dict(request.args)
         args.update(t_dict)
         args.update(request.form)
-        action = request.args.get('Action')
+        if args.get('param'):
+            try:
+                args.update(json5.loads(args.get('param')))
+            except ValueError:
+                logger.info(f"忽略param：{args.get('param')}")
+        if args.get('Action'):
+            args['action'] = args.get('Action')
+        action = args.get('action')
         # allow_action为空，表示没有设置允许的操作，则放行所有Action
-        if allow_action is None or action in allow_action:
+        if allow_action is None or action is None or action in allow_action:
             if use_action:
                 if len(args) == 0:
                     global example
@@ -92,13 +106,17 @@ def start_server_app(deal=None, app_name='yangke.web.flaskserver', instance_path
                                     "Info": "服务正常，请使用Action传入指定的操作！",
                                     "Example": example})
                 else:
-                    logger.debug("执行操作{}".format(action))
+                    logger.debug(f"执行操作{action}")
                     logger.debug(args)
             else:  # 没有使用Action，则直接放行，将所有请求转发至deal函数
                 pass
             try:
                 # result = eval("{}(args)".format(action))
                 result = deal(args) or {}  # 需要确保result不为空
+            except KeyError:
+                traceback.print_exc()
+                logger.debug(f"远程调用了{action}方法，但传递的参数错误！")
+                return jsonify({"Success": False})
             except:
                 traceback.print_exc()
                 if use_action:
@@ -128,8 +146,8 @@ def start_server_app(deal=None, app_name='yangke.web.flaskserver', instance_path
         return jsonify({"Success": True, "Info": f"通过 http://localhost:{port}/download/baidu.py 下载文件",
                         "Example": example})
 
+    # @cross_origin(origins="*", methods=['GET', 'POST'])
     @app.route('/download/<file_name>', methods=['GET', 'POST'])
-    @cross_origin(origins="*", methods=['GET', 'POST'])
     def download_file(file_name):
         """
         下载文件功能，该功能默认启动。通过 ip:port/download/<file_name> 下载文件
@@ -163,8 +181,8 @@ def start_server_app(deal=None, app_name='yangke.web.flaskserver', instance_path
         response = make_response(send_from_directory(directory, file_name, as_attachment=True))
         return response
 
+    # @cross_origin(origins="*", methods=['GET', 'POST'])
     @app.route('/preview/<file_name>', methods=['GET', 'POST'])
-    @cross_origin(origins="*", methods=['GET', 'POST'])
     def preview_file(file_name):
         """
         在线预览文件，暂不支持
@@ -180,9 +198,13 @@ def start_server_app(deal=None, app_name='yangke.web.flaskserver', instance_path
     else:
         app.add_url_rule("/", endpoint, start_service_null, **options)
     if run_immediate:  # 如果需要立即运行
-        logger.info(f"服务已启动，通过 http://localhost:{port}/ 访问")
-        from waitress import serve
-        serve(app, host=host, port=port)
+        if single_thread:
+            app.run(threaded=False, host=host, port=int(port))
+        else:
+            logger.info(f"服务已启动，通过 http://localhost:{port}/ 访问")
+            from waitress import serve
+            serve(app, host=host, port=port)
+
     return app
 
 
